import { PROP_CUSTOMIZE_TYPE } from "./constants"; // Option 1 - override property as common

export default function customizeProps(_ref) {
  let {
    type = PROP_CUSTOMIZE_TYPE.allProps,
    override = false,
    propNames = [],
    oldProps = {},
    newProps = {}
  } = _ref;

  switch (type) {
    case PROP_CUSTOMIZE_TYPE.allProps:
      return customizeAllProps({
        override,
        oldProps,
        newProps
      });

    case PROP_CUSTOMIZE_TYPE.selectedProps:
      return customizeSelectedProps({
        override,
        propNames,
        oldProps,
        newProps
      });

    case PROP_CUSTOMIZE_TYPE.excludedProps:
      return customizeAllPropsOtherThanExcluded({
        override,
        propNames,
        oldProps,
        newProps
      });

    default:
      return oldProps;
  }
}

function customizeAllProps(_ref2) {
  let {
    override,
    oldProps,
    newProps
  } = _ref2;

  if (override) {
    return { ...oldProps,
      ...newProps
    };
  } else {
    return { ...newProps,
      ...oldProps
    };
  }
}

function customizeSelectedProps(_ref3) {
  let {
    override,
    propNames,
    oldProps,
    newProps
  } = _ref3;
  return Object.keys(newProps).reduce((value, item) => {
    if (propNames.includes(item)) {
      if (override) {
        value[item] = newProps[item];
      } else {
        if (!value.hasOwnProperty(item)) {
          value[item] = newProps[item];
        }
      }
    }

    return value;
  }, { ...oldProps
  });
}

function customizeAllPropsOtherThanExcluded(_ref4) {
  let {
    override,
    propNames,
    oldProps,
    newProps
  } = _ref4;
  return Object.keys(newProps).reduce((value, item) => {
    if (!propNames.includes(item)) {
      if (override) {
        value[item] = newProps[item];
      } else {
        if (!value.hasOwnProperty(item)) {
          value[item] = newProps[item];
        }
      }
    }

    return value;
  }, { ...oldProps
  });
} // Option 2 - Override as symbol for each property that needs.
// export default function customizeProps({
//     type = PROP_CUSTOMIZE_TYPE.allProps,
//     propNames = [],
//     oldProps = {},
//     newProps = {}
// }) {
//     switch(type) {
//         case PROP_CUSTOMIZE_TYPE.allProps:
//             function isCustomizable() {
//                 return true;
//             }
//             return transformProps({
//                 isCustomizable,
//                 oldProps,
//                 newProps
//             });
//         case PROP_CUSTOMIZE_TYPE.selectedProps:
//             function isCustomizable(prop) {
//                 return propNames.includes(prop);
//             }
//             return transformProps({
//                 isCustomizable,
//                 oldProps,
//                 newProps
//             });
//         case PROP_CUSTOMIZE_TYPE.excludedProps:
//             function isCustomizable(prop) {
//                 return !propNames.includes(prop);
//             }
//             return transformProps({
//                 isCustomizable,
//                 oldProps,
//                 newProps
//             });
//         default:
//             return oldProps;
//     }
// }
// function transformProps({
//     isCustomizable,
//     oldProps,
//     newProps
// }) {
//     return Object.keys(newProps).reduce((value, item) => {
//         if(isCustomizable(item)) {
//             let override = item.startsWith('$') && item.endsWith('$');
//             if(override) {
//                 item = item.slice(1, item.length - 1);
//                 value[item] = newProps[item];
//             } else {
//                 if(!value.hasOwnProperty(item)) {
//                     value[item] = newProps[item];
//                 }
//             }
//         }
//         return value;
//     }, {...oldProps});
// }