import { PROP_CUSTOMIZE_TYPE } from "./constants";
export default function customizeProps(_ref) {
  let {
    type = PROP_CUSTOMIZE_TYPE.allProps,
    canOverride = false,
    propNames = [],
    oldProps = {},
    newProps = {}
  } = _ref;

  switch (type) {
    case PROP_CUSTOMIZE_TYPE.allProps:
      return customizeAllProps({
        canOverride,
        oldProps,
        newProps
      });

    case PROP_CUSTOMIZE_TYPE.selectedProps:
      return customizeSelectedProps({
        canOverride,
        propNames,
        oldProps,
        newProps
      });

    case PROP_CUSTOMIZE_TYPE.excludedProps:
      return customizeAllPropsOtherThanExcluded({
        canOverride,
        propNames,
        oldProps,
        newProps
      });

    default:
      return oldProps;
  }
}

function customizeAllProps(_ref2) {
  let {
    canOverride,
    oldProps,
    newProps
  } = _ref2;

  if (canOverride) {
    return { ...oldProps,
      ...newProps
    };
  } else {
    return { ...newProps,
      ...oldProps
    };
  }
}

function customizeSelectedProps(_ref3) {
  let {
    canOverride,
    propNames,
    oldProps,
    newProps
  } = _ref3;
  return Object.keys(newProps).reduce((value, item) => {
    if (propNames.includes(item)) {
      if (canOverride) {
        value[item] = newProps[item];
      } else {
        if (!value.hasOwnProperty(item)) {
          value[item] = newProps[item];
        }
      }
    }

    return value;
  }, { ...oldProps
  });
}

function customizeAllPropsOtherThanExcluded(_ref4) {
  let {
    canOverride,
    propNames,
    oldProps,
    newProps
  } = _ref4;
  return Object.keys(newProps).reduce((value, item) => {
    if (!propNames.includes(item)) {
      if (canOverride) {
        value[item] = newProps[item];
      } else {
        if (!value.hasOwnProperty(item)) {
          value[item] = newProps[item];
        }
      }
    }

    return value;
  }, { ...oldProps
  });
}