import React, { useCallback } from 'react';
import { ContextOptimizer } from "../Responsive/helpers/ContextOptimizer";
import ComponentCustomizerContext from "./context/ComponentCustomizerContext";
import customizeProps from "../../../helpers/component-customizer/customize-props"; // Option 1 - override property as common

export default function ComponentCustomizer(OrignalComponent, options) {
  function _ComponentCustomizer(props) {
    const getComponentProperties = useCallback(contextValue => {
      const {
        name: componentName
      } = options;
      return contextValue[componentName];
    }, []);
    return /*#__PURE__*/React.createElement(ContextOptimizer, {
      Context: ComponentCustomizerContext,
      calculation: getComponentProperties
    }, componentData => {
      if (!componentData) {
        return /*#__PURE__*/React.createElement(OrignalComponent, { ...props
        });
      }

      const {
        Component: NewComponent,
        override,
        props: newProps
      } = componentData;
      const {
        type,
        props: propNames
      } = options;
      const updatedProps = customizeProps({
        type,
        override,
        propNames,
        oldProps: props,
        newProps
      });
      const Component = NewComponent || OrignalComponent;
      return /*#__PURE__*/React.createElement(Component, { ...updatedProps
      });
    });
  }

  _ComponentCustomizer.displayName = 'ComponentCustomizer';
  return _ComponentCustomizer;
} // Option 2 - Override as symbol for each property that needs.
// export default function ComponentCustomizer(OrignalComponent, options) {
//     function _ComponentCustomizer(props) {
//         const getComponentProperties = useCallback((contextValue) => {
//             const {
//                 name: componentName
//             } = options;
//             return contextValue[componentName];
//         }, []);
//         return (
//             <ContextOptimizer
//                 Context={ComponentCustomizerContext}
//                 calculation={getComponentProperties}
//             >
//                 {(componentData) => {
//                     if(!componentData) {
//                         return <OrignalComponent {...props} />
//                     }
//                     const {
//                         Component: NewComponent,
//                         props: newProps
//                     } = componentData;
//                     const {
//                         type,
//                         props: propNames
//                     } = options;
//                     const updatedProps = customizeProps({
//                         type,
//                         propNames,
//                         oldProps: props,
//                         newProps
//                     });
//                     const Component = NewComponent || OrignalComponent;
//                     return <Component {...updatedProps} />
//                 }}
//             </ContextOptimizer>
//         );
//     }
//     _ComponentCustomizer.displayName = 'ComponentCustomizer';
//     return _ComponentCustomizer;
// }