# PropertyCustomizer

It is used to customize the component properties from the provider. Property override option also provided.

### PropertyCustomizationProvider

This is the provider that customize the component properties by sending the component name and props data. For each components, we have two options.

- **canOverride** - A boolean which will override the component properties send from any where. Default value is **false**.
- **props** - These are the properties object those send to the components.

**Usage:**

```jsx
<PropertyCustomizationProvider
  value={{
    Popup: {
      canOverride: true,
      props: {
        isAbsolutePositioningNeeded: false,
        isOutsideScrollBlocked: true
      }
    },
    Button: {
      canOverride: false,
      props: {
        size: 'large'
      }
    }
  }}
>
  {/* Components rendering */}
</PropertyCustomizationProvider>
```

### PropertyCustomizer

This is the HOC component that implemented for the component's property customization. This is having two arguments.

1. **Component:** The component which needs the property customization. (Required)
2. **Options:** This obejct contains below properties
  - **name:** A unique name for the component to get the props data from the Provider's entry.
  - **type:** Which props are going to be customized. It accepts one of these values `allProps` (Default), `selectedProps`, `excludedProps`.
  - **props:** Array of prop names. This works based on the above `type` property. If the type is,
    - `allProps` - This option will be skipped.
    - `selectedProps` - Only these props will be customizable.
    - `excludedProps` - These props will be excluded. All other props will be customizable.

**Usage:**

```jsx
function Popup(props) {
  /* Component implementation*/
}

PropertyCustomizer(Popup, {
  name: 'Popup',
  type: 'selectedProps',
  props: ['isAbsolutePositioningNeeded', 'isOutsideScrollBlocked']
});
```

```jsx
function Button(props) {
  /* Component implementation*/
}

PropertyCustomizer(Popup, {
  name: 'Button',
  type: 'excludedProps',
  props: ['children', 'text']
});
```