# ComponentCustomizer

It is used to customize the component and its properties from the provider. Property override option also implemented.

### ComponentCustomizationProvider

This is the provider that customize the component and it's properties by sending a object mapping of the component name and data. For each components, we have three options.

- **Component** - This component will be rendered instead of default component (Like as registry).
- **props** - This is the props object those send to the component.
- **override** - A boolean which decides that above given properties should override or default props. Default value is **false**.
  - If `true`, Given props will be getting priotized over props given from usage places.
  - If `false`, Given props will be working as component's default props (fallback).
- By default, all the given props are behave like default props. If any prop should override, we can add `$` at the start and end of that prop names. This works like `!important` from css.

**Usage:**

```jsx
<ComponentCustomizationProvider
  value={{
    Button: {
      Component: NewButton,
      props: {
        size: 'large',
        $rounded$: true
      },
      override: false
    },
    Popup: {
      props: {
        isAbsolutePositioningNeeded: false,
        isOutsideScrollBlocked: true
      },
      override: true
    }
  }}
>
  {/* App components rendering */}
</ComponentCustomizationProvider>
```

### ComponentCustomizer

This is the HOC component that implemented for the component and it's property customization. This is having two arguments.

1. **Component:** The component which needs the customization. (Required)
2. **Options:** This obejct contains below properties
  - **name:** A unique name for the component to get the customize data from the Provider. (Required)
  - **type:** It accepts one of these values `allProps` (Default), `selectedProps`, `excludedProps`. It defines which props are going to be customized.
  - **props:** Array of prop names. This works based on the above `type` property. If the type is,
    - `allProps` - This option will be skipped.
    - `selectedProps` - Only these props will be customizable.
    - `excludedProps` - These props will be excluded. All other props will be customizable.

**Usage:**

```jsx
function Button(props) {
  /* Component implementation*/
}

PropertyCustomizer(Popup, {
  name: 'Button',
  type: 'excludedProps',
  props: ['children', 'text']
});
```

```jsx
function Popup(props) {
  /* Component implementation*/
}

PropertyCustomizer(Popup, {
  name: 'Popup',
  type: 'selectedProps',
  props: ['isAbsolutePositioningNeeded', 'isOutsideScrollBlocked']
});
```